// src/clipboard.anatomy.ts
import { createAnatomy } from "@zag-js/anatomy";
var anatomy = createAnatomy("clipboard").parts("root", "control", "trigger", "indicator", "input", "label");
var parts = anatomy.build();

// src/clipboard.connect.ts
import { dataAttr } from "@zag-js/dom-query";

// src/clipboard.dom.ts
import { createScope, getWindow } from "@zag-js/dom-query";
import { hasProp } from "@zag-js/utils";
var dom = createScope({
  getRootId: (ctx) => ctx.ids?.root ?? `clip-${ctx.id}`,
  getInputId: (ctx) => ctx.ids?.input ?? `clip-input-${ctx.id}`,
  getLabelId: (ctx) => ctx.ids?.label ?? `clip-label-${ctx.id}`,
  getInputEl: (ctx) => dom.getById(ctx, dom.getInputId(ctx)),
  writeToClipboard: (ctx) => copyText(dom.getDoc(ctx), ctx.value)
});
function createNode(doc, text) {
  const node = doc.createElement("pre");
  Object.assign(node.style, {
    width: "1px",
    height: "1px",
    position: "fixed",
    top: "5px"
  });
  node.textContent = text;
  return node;
}
function copyNode(node) {
  const win = getWindow(node);
  const selection = win.getSelection();
  if (selection == null) {
    return Promise.reject(new Error());
  }
  selection.removeAllRanges();
  const doc = node.ownerDocument;
  const range = doc.createRange();
  range.selectNodeContents(node);
  selection.addRange(range);
  doc.execCommand("copy");
  selection.removeAllRanges();
  return Promise.resolve();
}
function copyText(doc, text) {
  const win = doc.defaultView || window;
  if (hasProp(win.navigator, "clipboard")) {
    return win.navigator.clipboard.writeText(text);
  }
  if (!doc.body) {
    return Promise.reject(new Error());
  }
  const node = createNode(doc, text);
  doc.body.appendChild(node);
  copyNode(node);
  doc.body.removeChild(node);
  return Promise.resolve();
}

// src/clipboard.connect.ts
function connect(state, send, normalize) {
  const isCopied = state.matches("copied");
  return {
    isCopied,
    value: state.context.value,
    setValue(value) {
      send({ type: "VALUE.SET", value });
    },
    copy() {
      send({ type: "COPY" });
    },
    rootProps: normalize.element({
      ...parts.root.attrs,
      "data-copied": dataAttr(isCopied),
      id: dom.getRootId(state.context)
    }),
    labelProps: normalize.label({
      ...parts.label.attrs,
      htmlFor: dom.getInputId(state.context),
      "data-copied": dataAttr(isCopied),
      id: dom.getLabelId(state.context)
    }),
    controlProps: normalize.element({
      ...parts.control.attrs,
      "data-copied": dataAttr(isCopied)
    }),
    inputProps: normalize.input({
      ...parts.input.attrs,
      defaultValue: state.context.value,
      "data-copied": dataAttr(isCopied),
      readOnly: true,
      "data-readonly": "true",
      id: dom.getInputId(state.context),
      onFocus(event) {
        event.currentTarget.select();
      },
      onCopy() {
        send({ type: "INPUT.COPY" });
      }
    }),
    triggerProps: normalize.button({
      ...parts.trigger.attrs,
      "aria-label": isCopied ? "Copied to clipboard" : "Copy to clipboard",
      "data-copied": dataAttr(isCopied),
      onClick() {
        send({ type: "COPY" });
      }
    }),
    getIndicatorProps(props2) {
      return normalize.element({
        ...parts.indicator.attrs,
        hidden: props2.copied !== isCopied
      });
    }
  };
}

// src/clipboard.machine.ts
import { createMachine } from "@zag-js/core";
import { compact } from "@zag-js/utils";
function machine(userContext) {
  const ctx = compact(userContext);
  return createMachine(
    {
      id: "clipboard",
      initial: "idle",
      context: {
        value: "",
        timeout: 3e3,
        ...ctx
      },
      watch: {
        value: ["syncInputElement"]
      },
      on: {
        "VALUE.SET": {
          actions: ["setValue"]
        },
        COPY: {
          target: "copied",
          actions: ["copyToClipboard", "invokeOnCopied"]
        }
      },
      states: {
        idle: {
          on: {
            "INPUT.COPY": {
              target: "copied",
              actions: ["invokeOnCopied"]
            }
          }
        },
        copied: {
          after: {
            COPY_TIMEOUT: "idle"
          },
          on: {
            COPY: {
              target: "copied",
              actions: ["copyToClipboard", "invokeOnCopied"]
            },
            "INPUT.COPY": {
              actions: ["invokeOnCopied"]
            }
          }
        }
      }
    },
    {
      actions: {
        setValue(ctx2, evt) {
          ctx2.value = evt.value;
        },
        copyToClipboard(ctx2) {
          dom.writeToClipboard(ctx2);
        },
        invokeOnCopied(ctx2) {
          ctx2.onCopyStatusChange?.({ copied: true });
        },
        syncInputElement(ctx2) {
          dom.setValue(dom.getInputEl(ctx2), ctx2.value);
        }
      },
      delays: {
        COPY_TIMEOUT: (ctx2) => ctx2.timeout
      }
    }
  );
}

// src/clipboard.props.ts
import { createProps } from "@zag-js/types";
import { createSplitProps } from "@zag-js/utils";
var props = createProps()([
  "getRootNode",
  "id",
  "ids",
  "value",
  "timeout",
  "onCopyStatusChange"
]);
var contextProps = createSplitProps(props);
var indicatorProps = createProps()(["copied"]);
var splitIndicatorProps = createSplitProps(indicatorProps);
export {
  anatomy,
  connect,
  contextProps,
  indicatorProps,
  machine,
  props,
  splitIndicatorProps
};
//# sourceMappingURL=index.mjs.map